<!DOCTYPE html>
<html>
<head>
  <title>InvisibleCaptcha Demo</title>
  <%= stylesheet_link_tag "/styles.css" %>
  <%= csrf_meta_tags %>
  <%= invisible_captcha_styles %>
</head>
<body>
  <h1>InvisibleCaptcha v<%= InvisibleCaptcha::VERSION %> - Demo</h1>

  <p>
    <%= link_to "Default settings", new_topic_path %> |
    <%= link_to "With visual honeypots", new_topic_path(context: "visual_honeypots") %> |
    <%= link_to "With timestamp disabled", new_topic_path(context: "timestamp_disabled") %>
  </p>

  <% flash.each do |key, value| %>
    <ul class="errors">
      <li>
        <%= "[#{key.upcase}] #{value}" %>
      </li>
    </ul>
  <% end %>

  <%= yield %>

  <footer>
    Running on
    <b>Ruby <%= RUBY_VERSION %></b> and
    <b>Rails <%= Rails.version %></b>
  </footer>
  </body>
</html>
